# Author: Shirley Pu
# Code used for the paper "Establishing performance metrics for quantitative non-targeted analysis: a demonstration using per- and polyfluoroalkyl substances"
# Contains code used for data analysis

# Load libraries
library(tidyverse)
library(investr)
library(ggridges)
library(lme4)
library(lmerTest)

# Source custom functions
source("Pu_et_al_qNTA_metrics_Supplementary_File_2_scripts.R")

# Load data
# Code used to prepare data for R analysis is located in "Pu_et_al_qNTA_metrics_data.R"
load("Pu_et_al_qNTA_metrics_data.RData")

# Create vectors containing native and IS compounds
# They are in order of increasing retention time
# matched_native and matched_IS are used for Approach 1
matched_native<-c("PFBA","PFPeA","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","6:2 FTS","PFOA","PFHxS","PFNA","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFDoDA","PFOSA","PFTeDA")
matched_IS<-c("MPFBA","M5PFPeA","M2-4:2 FTS","M3PFBS","M5PFHxA","M3HFPO-DA","M4PFHpA","M2-6:2 FTS","M8PFOA","M3PFHxS","M9PFNA","M2-8:2 FTS","M6PFDA","d3-N-MeFOSAA","M8PFOS","d5-N-EtFOSAA","M7PFUnDA","M2PFDoDA","M8PFOSA","M2PFTeDA")
# all_targeted_native contains all 29 native chemicals included in the study (used for Approach 2)
all_targeted_native<-c("PFMOAA","PFBA","PFECA-F","PFPeA","PFECA-A","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","PFPeS","6:2 FTS","PFOA","PFHxS","NBP2","PFNA","PFHpS","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFNS","PFDoDA","PFDS","PFTrDA","PFOSA","PFTeDA")
# all_approaches_native chemicals are those with measurements available for both targeted acquisition (XCalibur Quan) and non-targeted acquisition (Compound Discoverer). n=26 (no PFMOAA, PFBA, PFECA-F because below signal threshold)
all_approaches_native<-c("PFPeA","PFECA-A","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","PFPeS","6:2 FTS","PFOA","PFHxS","NBP2","PFNA","PFHpS","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFNS","PFDoDA","PFDS","PFTrDA","PFOSA","PFTeDA")

### CALIBRATION CURVES### 
## Results- 4.1 ##
## Targeted acquisition (Xcalibur Quan)
A1_lm_params<-data.frame(slope=vector(mode="numeric",length=length(matched_native)),R_squared=vector(mode="numeric",length=length(matched_native)))
A2_lm_params<-data.frame(slope=vector(mode="numeric",length=length(all_targeted_native)),R_squared=vector(mode="numeric",length=length(all_targeted_native)))
A3_lm_params<-data.frame(slope=vector(mode="numeric",length=length(all_approaches_native)),R_squared=vector(mode="numeric",length=length(all_approaches_native)))

# Approach 1: Normalized ion abundances
for(i in 1:length(matched_native)){
  curr_A1_lm<-ggplot_calcurve(matched_native[i],matched_IS[i],XQuan_wide,ratio=TRUE,plot_out=FALSE,save=TRUE)
  A1_lm_params$slope[[i]]<-curr_A1_lm$coefficients[2]
  A1_lm_params$R_squared[[i]]<-summary(curr_A1_lm)$r.squared
}

# Approach 2: Raw ion abundances
for(i in 1:length(all_targeted_native)){
  curr_A2_lm<-ggplot_calcurve(all_targeted_native[i],all_targeted_native[i],XQuan_wide,ratio=FALSE,plot_out=FALSE,save=TRUE)
  A2_lm_params$slope[[i]]<-curr_A2_lm$coefficients[2]
  A2_lm_params$R_squared[[i]]<-summary(curr_A2_lm)$r.squared
}

## Non-targeted acquisition
# Approach 3: Raw ion abundance
for(i in 1:length(all_approaches_native)){
  curr_A3_lm<-ggplot_calcurve(all_approaches_native[i],all_approaches_native[i],CD_areas,ratio=FALSE,plot_out=FALSE,save=TRUE)
  A3_lm_params$slope[[i]]<-curr_A3_lm$coefficients[2]
  A3_lm_params$R_squared[[i]]<-summary(curr_A3_lm)$r.squared
}

# calibration curve parameters
mean(A1_lm_params$slope); range(A1_lm_params$slope); mean(A1_lm_params$R_squared); range(A1_lm_params$R_squared)
mean(A2_lm_params$slope); range(A2_lm_params$slope); mean(A2_lm_params$R_squared); range(A2_lm_params$R_squared)
mean(A3_lm_params$slope); range(A3_lm_params$slope); mean(A3_lm_params$R_squared); range(A3_lm_params$R_squared)

# t-test for slope=1, A1-A3 
t.test(A1_lm_params$slope,mu=1)
t.test(A2_lm_params$slope,mu=1)
t.test(A3_lm_params$slope,mu=1)

### INVERSE PREDICTION ###
## Targeted acquisition (Xcalibur Quan)
# Approach 1: Normalized ion abundances
A1_invpred<-data.frame()
for(i in 1:length(matched_native)){
  A1_invpred<-rbind(A1_invpred,get_invest_leaveout(matched_native[i],matched_IS[i],XQuan_wide,ratio=TRUE,as_df=TRUE))
}
A1_invpred$lower_exceeded<-A1_invpred$Exp.Amt<A1_invpred$lower
A1_invpred$upper_exceeded<-A1_invpred$Exp.Amt>A1_invpred$upper
A1_invpred$either_exceeded<-A1_invpred$Exp.Amt<A1_invpred$lower|A1_invpred$Exp.Amt>A1_invpred$upper

# Approach 2: Raw ion abundances
A2_invpred<-data.frame()
for(i in 1:length(all_targeted_native)){
  A2_invpred<-rbind(A2_invpred,get_invest_leaveout(all_targeted_native[i],all_targeted_native[i],XQuan_wide,ratio=FALSE,as_df=TRUE))
}
A2_invpred$lower_exceeded<-A2_invpred$Exp.Amt<A2_invpred$lower
A2_invpred$upper_exceeded<-A2_invpred$Exp.Amt>A2_invpred$upper
A2_invpred$either_exceeded<-A2_invpred$Exp.Amt<A2_invpred$lower|A2_invpred$Exp.Amt>A2_invpred$upper

## Non-targeted acquisition (Compound Discoverer)
# Approach 3: Raw ion abundances
A3_invpred<-data.frame()
for(i in 1:length(all_approaches_native)){
  A3_invpred<-rbind(A3_invpred,get_invest_leaveout(all_approaches_native[i],all_approaches_native[i],CD_areas,ratio=FALSE,as_df=TRUE))
}
A3_invpred$lower_exceeded<-A3_invpred$Exp.Amt<A3_invpred$lower
A3_invpred$upper_exceeded<-A3_invpred$Exp.Amt>A3_invpred$upper
A3_invpred$either_exceeded<-A3_invpred$Exp.Amt<A3_invpred$lower|A3_invpred$Exp.Amt>A3_invpred$upper

### RESPONSE FACTOR DISTRIBUTIONS ### 
## Results- 4.2 ##
# Get RFs (these also have an Area column)
XQ_RFs<-data.frame()
for(i in 1:length(all_targeted_native)){
  XQ_RFs<-rbind(XQ_RFs,get_RF(all_targeted_native[i],all_targeted_native[i],XQuan_wide,ratio=FALSE))
}
XQ_RFs<-XQ_RFs[complete.cases(XQ_RFs),]
XQ_RFs<-dplyr::left_join(XQ_RFs,PFAS_solvent_only_RT,by=c("native"="Compound"))

CD_RFs<-data.frame()
for(i in 1:length(all_approaches_native)){
  CD_RFs<-rbind(CD_RFs,get_RF(all_approaches_native[i],all_approaches_native[i],CD_areas,ratio=FALSE))
}
CD_RFs<-CD_RFs[complete.cases(CD_RFs),]
CD_RFs<-dplyr::left_join(CD_RFs,PFAS_solvent_only_RT,by=c("native"="Compound"))

# RF statistics for NTA data
CD_RFs_medians<-CD_RFs %>% group_by(native) %>% summarize_at('RF',median,na.rm=TRUE)
min(CD_RFs_medians$RF)
max(CD_RFs_medians$RF)
max(CD_RFs_medians$RF)/min(CD_RFs_medians$RF)

max(CD_RFs$RF)/min(CD_RFs$RF)

## Figure 2 ##
# Compound Discoverer RF distributions
# Adjust joint bandwidth (multimodality of some RF distributions may result from the bandwidth size)
ggplot(CD_RFs,aes(x=RF,y=reorder(native,-RT)))+
  xlab(expression('Response Factor (ng/mL)'^-1))+
  ylab("Chemical")+
  geom_density_ridges(scale=2,alpha=.5,bandwidth=0.1)+
  scale_x_log10(breaks = scales::trans_breaks("log10", function(x) 10^x),
                labels = scales::trans_format("log10", scales::math_format(10^.x)))+
  theme_bw(base_size=22)

### BOUNDED RESPONSE FACTOR BOOTSTRAP qNTA APPROACHES ###
## Approach 4: Expert-selected surrogates
# takes ~12 minutes with 3 surrogates per compound, 26 compounds, 10,000 runs

# Do bootstrap resampling
set.seed(1)
A4_expert_boot<-data.frame()
for(i in 1:nrow(expert_surrogates)){
  A4_expert_boot<-rbind(A4_expert_boot,cbind(Compound=expert_surrogates$Compound[i],full_boot(CD_RFs[CD_RFs$native %in% expert_surrogates[i,2:4],],runs=10000,med_out=FALSE,resample_size=26)))
}

# Calculate point predictions
A4_expert_boot_preds<-dplyr::left_join(CD_RFs %>% select(native,Exp.Amt,Area,replicate),A4_expert_boot[,c("Compound","med_2.5th_percentile","med_50th_percentile","med_97.5th_percentile")],by=c("native"="Compound")) %>% 
  mutate(estimate=Area/med_50th_percentile,
         lower=Area/med_97.5th_percentile,
         upper=Area/med_2.5th_percentile,
         AQ=10^(log10(estimate/Exp.Amt)),
         AAQ=10^abs(log10(estimate/Exp.Amt)),
         CLFR=upper/lower,
         LCQ=Exp.Amt/lower,
         UCQ=upper/Exp.Amt,
         lower_exceeded=Exp.Amt<lower,
         upper_exceeded=Exp.Amt>upper,
         either_exceeded=Exp.Amt<lower|Exp.Amt>upper
         )

## Approach 5: Global surrogates
# takes ~20 minutes with 10,000 runs and 26 compounds
set.seed(1)
A5_global_boot<-data.frame()
A5_global_boot<-loocv_new(CD_RFs,runs=10000,med_out=FALSE)
A5_global_boot$Compound<-unique(CD_RFs$native)
A5_global_boot_preds<-dplyr::left_join(CD_RFs %>% select(native,Exp.Amt,Area,replicate),A5_global_boot[,c("Compound","med_2.5th_percentile","med_50th_percentile","med_97.5th_percentile")],by=c("native"="Compound")) %>% 
  mutate(estimate=Area/med_50th_percentile,
         lower=Area/med_97.5th_percentile,
         upper=Area/med_2.5th_percentile,
         AQ=10^(log10(estimate/Exp.Amt)),
         AAQ=10^abs(log10(estimate/Exp.Amt)),
         CLFR=upper/lower,
         LCQ=Exp.Amt/lower,
         UCQ=upper/Exp.Amt,
         lower_exceeded=Exp.Amt<lower,
         upper_exceeded=Exp.Amt>upper,
         either_exceeded=Exp.Amt<lower|Exp.Amt>upper
         )

### PREPARING ALL MODEL PERFORMANCE METRICS FOR SUPPLEMENTAL MATERIAL
# get normalized areas to add to dataframe
XQ_RRFs<-data.frame()
for(i in 1:length(matched_native)){
  XQ_RRFs<-rbind(XQ_RRFs,get_RF(matched_native[i],matched_IS[i],XQuan_wide,ratio=TRUE))
}
XQ_RRFs<-XQ_RRFs[complete.cases(XQ_RRFs),]

# make one combined dataframe
all_approaches_results<-XQ_RFs %>% 
  select(native,Exp.Amt,replicate,Area,RT) %>% 
  rename(XQ_Area=Area) %>% 
  dplyr::left_join(XQ_RRFs %>% select(-RRF) %>% rename(XQ_Area_norm=Response.Ratio)) %>%
  dplyr::left_join(CD_RFs %>% select(-c(RF,IS,RT)) %>% rename(CD_Area=Area)) %>%
  select(-RT)

approach_list<-list(A1=A1_invpred %>% rename_with(~gsub("_Cmp","",.x)) %>% select(-IS),A2=A2_invpred %>% rename_with(~gsub("_Cmp","",.x)) %>% select(-IS),A3=A3_invpred %>% rename_with(~gsub("_Cmp","",.x)) %>% select(-IS),A4=(A4_expert_boot_preds %>% select(-contains("perc"))),A5=(A5_global_boot_preds %>% select(-contains("perc"))))
for(i in 1:5){
  curr_approach<-approach_list[[i]] %>% select(-any_of(c("Area","in_interval","logspace_estimate", "logspace_lower", "logspace_upper","slope","Rsquared")))
  all_approaches_results<-dplyr::left_join(all_approaches_results,curr_approach %>% rename_with(~paste0(.x,"_A",i),.cols=c(estimate,lower,upper,AQ,AAQ,CLFR,LCQ,UCQ,lower_exceeded,upper_exceeded,either_exceeded)),by=c("native"="native","Exp.Amt"="Exp.Amt","replicate"="replicate"))
}

all_approaches_results_long<-data.frame()
for(i in 1:5){
  curr_approach<-approach_list[[i]] %>% select(-any_of(c("IS","Area","in_interval", "logspace_estimate", "logspace_lower", "logspace_upper", "logspace_interval", "slope", "Rsquared"))) %>% 
    mutate(approach=names(approach_list)[i])
  all_approaches_results_long<-rbind(all_approaches_results_long,curr_approach)
}

# Table S3
# sorted by name (alphabetical, top to bottom), Exp.Amt, replicate
all_approaches_results_SI<-all_approaches_results %>%
  replace(is.na(.),"") %>%
  arrange(native,Exp.Amt,replicate)

### COMPARING APPROACHES- ACCURACY, UNCERTAINTY, RELIABILITY ### 
## Results- 4.3 ##
## Accuracy
# Accuracy Quotient (AQ) ranges for A1-A5
range(A1_invpred$AQ)
range(A2_invpred$AQ)
range(A3_invpred$AQ)
range(A4_expert_boot_preds$AQ)
range(A5_global_boot_preds$AQ)

# Percent of AQ between 0.7 and 1.3 (~30% relative error or less)
(A1_invpred %>% filter(abs(AQ-1)<=0.3) %>% nrow())/nrow(A1_invpred)
(A2_invpred %>% filter(abs(AQ-1)<=0.3) %>% nrow())/nrow(A2_invpred)
(A3_invpred %>% filter(abs(AQ-1)<=0.3) %>% nrow())/nrow(A3_invpred)
(A4_expert_boot_preds %>% filter(abs(AQ-1)<=0.3) %>% nrow())/nrow(A4_expert_boot_preds)
(A5_global_boot_preds %>% filter(abs(AQ-1)<=0.3) %>% nrow())/nrow(A5_global_boot_preds)

# Median AQ values
median(A1_invpred$AQ)
median(A2_invpred$AQ)
median(A3_invpred$AQ)
median(A4_expert_boot_preds$AQ)
median(A5_global_boot_preds$AQ)

# Min Absolute Accuracy Quotient (AAQ)
min(all_approaches_results_long$AAQ)
# Max Accuracy Quotient
max(all_approaches_results_long$AAQ)

# Median Absolute Accuracy Quotient
median(A1_invpred %>% pull('AAQ'))
median(A2_invpred %>% pull('AAQ'))
median(A3_invpred %>% pull('AAQ'))
median(A4_expert_boot_preds %>% pull('AAQ'))
median(A5_global_boot_preds %>% pull('AAQ'))

# Ninety-fifth percentile AAQ values
quantile(A1_invpred$AAQ,0.95)
quantile(A2_invpred$AAQ,0.95)
quantile(A3_invpred$AAQ,0.95)
quantile(A4_expert_boot_preds$AAQ,0.95)
quantile(A5_global_boot_preds$AAQ,0.95)

## Confidence
# Confidence limit fold range (CLFR)
# Minimum CLFR 
min(all_approaches_results_long$CLFR)
# Maximum CLFR
max(all_approaches_results_long$CLFR)
# Median CLFR
median(A1_invpred$CLFR)
median(A2_invpred$CLFR)
median(A3_invpred$CLFR)
median(A4_expert_boot_preds$CLFR)
median(A5_global_boot_preds$CLFR)

# Ninety-fifth percentile CLFR
quantile(A1_invpred$CLFR,0.95)
quantile(A2_invpred$CLFR,0.95)
quantile(A3_invpred$CLFR,0.95)
quantile(A4_expert_boot_preds$CLFR,0.95)
quantile(A5_global_boot_preds$CLFR,0.95)

# Percent of CLFR <= 2.19
sum(A1_invpred$CLFR<=2.19)/nrow(A1_invpred)
sum(A2_invpred$CLFR<=2.19)/nrow(A2_invpred)
sum(A3_invpred$CLFR<=2.19)/nrow(A3_invpred)
sum(A4_expert_boot_preds$CLFR<=2.19)/nrow(A4_expert_boot_preds)
sum(A5_global_boot_preds$CLFR<=2.19)/nrow(A5_global_boot_preds)

## Upper Confidence Quotient (UCQ)
# Range
range(all_approaches_results_long$UCQ)

# Median Upper Confidence Quotient
median(A1_invpred$UCQ)
median(A2_invpred$UCQ)
median(A3_invpred$UCQ)
median(A4_expert_boot_preds$UCQ)
median(A5_global_boot_preds$UCQ)

# Ninety-fifth percentile UCQ
quantile(A1_invpred$UCQ,0.95)
quantile(A2_invpred$UCQ,0.95)
quantile(A3_invpred$UCQ,0.95)
quantile(A4_expert_boot_preds$UCQ,0.95)
quantile(A5_global_boot_preds$UCQ,0.95)

# Percent of UCQ values <= 10
sum(A1_invpred$UCQ<=10)/nrow(A1_invpred)
sum(A2_invpred$UCQ<=10)/nrow(A2_invpred)
sum(A3_invpred$UCQ<=10)/nrow(A3_invpred)
sum(A4_expert_boot_preds$UCQ<=10)/nrow(A4_expert_boot_preds)
sum(A5_global_boot_preds$UCQ<=10)/nrow(A5_global_boot_preds)

# Most extreme underpredictions (lowest UCQ)
min(A1_invpred$UCQ)
min(A2_invpred$UCQ)
min(A3_invpred$UCQ)
min(A4_expert_boot_preds$UCQ)
min(A5_global_boot_preds$UCQ)

## Reliability
## Overall Reliability Percentage (ORP)
sum(A1_invpred$upper>=A1_invpred$Exp.Amt & A1_invpred$lower<=A1_invpred$Exp.Amt)/nrow(A1_invpred)
sum(A2_invpred$upper>=A2_invpred$Exp.Amt & A2_invpred$lower<=A2_invpred$Exp.Amt)/nrow(A2_invpred)
sum(A3_invpred$upper>=A3_invpred$Exp.Amt & A3_invpred$lower<=A3_invpred$Exp.Amt)/nrow(A3_invpred)
sum(A4_expert_boot_preds$upper>=A4_expert_boot_preds$Exp.Amt & A4_expert_boot_preds$lower<=A4_expert_boot_preds$Exp.Amt)/nrow(A4_expert_boot_preds)
sum(A5_global_boot_preds$upper>=A5_global_boot_preds$Exp.Amt & A5_global_boot_preds$lower<=A5_global_boot_preds$Exp.Amt)/nrow(A5_global_boot_preds)

## Upper Reliability Percentage (URP)
sum(A1_invpred$upper>=A1_invpred$Exp.Amt)/nrow(A1_invpred)
sum(A2_invpred$upper>=A2_invpred$Exp.Amt)/nrow(A2_invpred)
sum(A3_invpred$upper>=A3_invpred$Exp.Amt)/nrow(A3_invpred)
sum(A4_expert_boot_preds$upper>=A4_expert_boot_preds$Exp.Amt)/nrow(A4_expert_boot_preds)
sum(A5_global_boot_preds$upper>=A5_global_boot_preds$Exp.Amt)/nrow(A5_global_boot_preds)

### ONE WAY RANDOM EFFECTS MODELS FOR DIFFERENCE IN MODEL PERFORMANCE METRICS###
## Table 1 ##
# Comparing pairwise difference in accuracy quotient (AQ) between Approaches 1, 2, 3, 4, 5 using linear mixed-model
approach_combns<-combn(x=c("A1","A2","A3","A4","A5"),m=2) %>% t()

# AAQ
approaches_AAQ<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with('estimate'),starts_with('AAQ'))
approaches_AAQ_lmer<-vector(mode="list",length=nrow(approach_combns))
AAQ_diff_lengths<-vector(mode="numeric",length=nrow(approach_combns))
for(i in 1:nrow(approach_combns)){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  AAQ_1<-paste0("AAQ_",pair1)
  AAQ_2<-paste0("AAQ_",pair2)
  AAQ_diff<-(approaches_AAQ %>% pull(AAQ_2) %>% log10())-(approaches_AAQ %>% pull(AAQ_1) %>% log10())
  AAQ_diff_lengths[i]<-length(AAQ_diff[!is.na(AAQ_diff)])
  AAQ_diff_name<-paste0("log_AQ_diff_",pair2,"_",pair1)
  approaches_AAQ<-approaches_AAQ %>% 
    mutate(!!AAQ_diff_name := AAQ_diff)
  curr_lmer<-lmer(paste0("`",AAQ_diff_name,"`","~(1|native)"), data=approaches_AAQ %>% filter(!is.na(.data[[AAQ_diff_name]])))
  approaches_AAQ_lmer[[i]]<-curr_lmer
  names(approaches_AAQ_lmer)[[i]]<-paste0(pair2,"_",pair1)
  }

approach_AAQ_lmer_coef<-sapply(approaches_AAQ_lmer,function(x) summary(x)$coefficients) %>% t()
colnames(approach_AAQ_lmer_coef)<-c("Estimate","Std.Error","df","t_value","p_value")
approach_AAQ_lmer_coef<-as.data.frame(approach_AAQ_lmer_coef) 
# Approximately the geometric mean fold difference
approach_AAQ_lmer_coef$GM_FD<-10^(approach_AAQ_lmer_coef$Estimate)
approach_AAQ_lmer_coef$n<-AAQ_diff_lengths

# Comparing difference in CLFR  between Approach 1, 2, 3, 4, 5 (pairs in order) using linear mixed-models
approaches_CLFR<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with("CLFR"))
approaches_CLFR_lmer<-vector(mode="list",length=nrow(approach_combns)-1)
CLFR_diff_lengths<-vector(mode="numeric",length=nrow(approach_combns)-1)
for(i in 1:(nrow(approach_combns)-1)){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  CLFR_1<-paste0("CLFR_",pair1)
  CLFR_2<-paste0("CLFR_",pair2)
  CLFR_diff<-(approaches_CLFR %>% pull(CLFR_2) %>% log10())-(approaches_CLFR %>% pull(CLFR_1) %>% log10())
  CLFR_diff_lengths[i]<-length(CLFR_diff[!is.na(CLFR_diff)])
  CLFR_diff_name<-paste0("log_CLFR_diff_",pair2,"_",pair1)
  approaches_CLFR<-approaches_CLFR %>% 
    mutate(!!CLFR_diff_name := CLFR_diff)
  curr_lmer<-lmer(paste0("`",CLFR_diff_name,"`","~(1|native)"), data=approaches_CLFR %>% filter(!is.na(.data[[CLFR_diff_name]])))
  approaches_CLFR_lmer[[i]]<-curr_lmer
  names(approaches_CLFR_lmer)[[i]]<-paste0(pair2,"_",pair1)
}

approach_CLFR_lmer_coef<-sapply(approaches_CLFR_lmer,function(x) summary(x)$coefficients) %>% t()
colnames(approach_CLFR_lmer_coef)<-c("Estimate","Std.Error","df","t_value","p_value")
approach_CLFR_lmer_coef<-as.data.frame(approach_CLFR_lmer_coef) 
# Approximately the geometric mean fold difference
approach_CLFR_lmer_coef$GM_FD<-10^(approach_CLFR_lmer_coef$Estimate)
approach_CLFR_lmer_coef$n<-CLFR_diff_lengths

# do Wilcoxon Rank-Sum test for CLFR A4-A5 comparison
# consolidate to one CLFR per chemical
approaches_CLFR_A4_A5<-approaches_CLFR %>% 
  group_by(native) %>% 
  summarize_at(c("CLFR_A4","CLFR_A5"),median,na.rm=TRUE) %>% 
  mutate(log_CLFR_diff_A4_A5=log10(CLFR_A5)-log10(CLFR_A4)) %>%
  drop_na()

approaches_CLFR_wilcox<-wilcox.test(approaches_CLFR_A4_A5 %>% pull("log_CLFR_diff_A4_A5"),conf.int=TRUE)
# the estimate is the pseudomedian
# 10^mu is using median in this case
approaches_CLFR_wilcox<-c(approaches_CLFR_wilcox,
                         # Fold difference from using median, not mean
                         median_FD=10^(approaches_CLFR_wilcox$estimate))
                                
# Comparing difference in upper confidence quotients (UCQ) between Approaches 1, 2, 3, 4, 5 (pairs in order) using linear mixed-model
approaches_UCQ<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with('UCQ'))
approaches_UCQ_lmer<-vector(mode="list",length=nrow(approach_combns)-1)
UCQ_diff_lengths<-vector(mode="numeric",length=nrow(approach_combns)-1)
for(i in 1:(nrow(approach_combns)-1)){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  UCQ_1<-paste0("UCQ_",pair1)
  UCQ_2<-paste0("UCQ_",pair2)
  UCQ_diff<-(approaches_UCQ %>% pull(UCQ_2) %>% log10())-(approaches_UCQ %>% pull(UCQ_1) %>% log10())
  UCQ_diff_name<-paste0("log_UCQ_diff_",pair1,"_",pair2)
  approaches_UCQ<-approaches_UCQ %>% 
    mutate(!!UCQ_diff_name := UCQ_diff)
  UCQ_diff_lengths[i]<-approaches_UCQ %>% filter(!!as.symbol(UCQ_1)>=1 & !!as.symbol(UCQ_2)>=1) %>% filter(!is.na(!!as.symbol(UCQ_diff_name))) %>% nrow()
  curr_lmer<-lmer(paste0("`",UCQ_diff_name,"`","~(1|native)"), data=approaches_UCQ %>% filter(.data[[UCQ_1]]>=1 & .data[[UCQ_2]]>=1) %>% filter(!is.na(.data[[UCQ_diff_name]])))
  approaches_UCQ_lmer[[i]]<-curr_lmer
  names(approaches_UCQ_lmer)[[i]]<-paste0(pair2,"_",pair1)
}

approach_UCQ_lmer_coef<-sapply(approaches_UCQ_lmer,function(x) summary(x)$coefficients) %>% t()
colnames(approach_UCQ_lmer_coef)<-c("Estimate","Std.Error","df","t_value","p_value")
approach_UCQ_lmer_coef<-as.data.frame(approach_UCQ_lmer_coef) %>% 
  # Approximately the geometric mean fold difference
  mutate(GM_FD=10^Estimate) %>%
  mutate(n=UCQ_diff_lengths)

# do Wilcoxon Rank-Sum test for UCQ A4-A5 comparison
# consolidate to one UCQ per chemical
approaches_UCQ_A4_A5<-approaches_UCQ %>% 
  group_by(native) %>% 
  summarize_at(c("UCQ_A4","UCQ_A5"),median,na.rm=TRUE) %>% 
  filter(UCQ_A4>=1 & UCQ_A5>=1) %>% 
  mutate(log_UCQ_diff_A4_A5=log10(UCQ_A5)-log10(UCQ_A4))

approaches_UCQ_wilcox<-wilcox.test(approaches_UCQ_A4_A5 %>% pull("log_UCQ_diff_A4_A5"),conf.int=TRUE)
approaches_UCQ_wilcox<-c(approaches_UCQ_wilcox,
                              median_FD=10^(approaches_UCQ_wilcox$estimate))


### PER-CHEMICAL APPROACH PERFORMANCE (A1-A5)
all_approaches_results_perchem<-all_approaches_results %>% 
  group_by(native) %>%
  summarize_at(vars(contains("AAQ_A"),contains("CLFR_A"),contains("UCQ_A"),contains("LCQ_A")), ~ median(abs(.x),na.rm=TRUE))
all_approaches_results_perchem<-dplyr::left_join(all_approaches_results_perchem,PFAS_solvent_only_RT,by=c("native"="Compound")) %>%
  arrange(RT)

all_approaches_results_perchem_RP<-all_approaches_results %>%
  group_by(native) %>%
  summarize_at(vars(contains("upper_exceeded"),contains("lower_exceeded"),contains("either_exceeded")), ~ 1 - (sum(.x[!is.na(.x)])/length(.x[!is.na(.x)]))) %>%
  rename_with(~gsub("upper_exceeded","URP",.x)) %>%
  rename_with(~gsub("lower_exceeded","LRP",.x)) %>%
  rename_with(~gsub("either_exceeded","ORP",.x)) %>%
  mutate_all(~ifelse(is.nan(.), NA, .))
all_approaches_results_perchem_RP<-dplyr::left_join(all_approaches_results_perchem_RP,PFAS_solvent_only_RT,by=c("native"="Compound")) %>%
  arrange(RT)

med_CD_RF_perchem<-all_approaches_results %>% 
  group_by(native) %>%
  mutate(RF=CD_Area/Exp.Amt) %>%
  summarize_at("RF",median,na.rm=TRUE) %>%
  rename(median_RF=RF)

## Table S5 ##
A1_table<-dplyr::left_join(cbind(data.frame(native=matched_native),A1_lm_params),all_approaches_results_perchem) %>%
  dplyr::left_join(all_approaches_results_perchem_RP) %>%
  select(native,AAQ_A1,CLFR_A1,UCQ_A1,LCQ_A1,URP_A1,LRP_A1,ORP_A1,slope,R_squared)
A2_table<-dplyr::left_join(cbind(data.frame(native=all_targeted_native),A2_lm_params),all_approaches_results_perchem) %>%
  dplyr::left_join(all_approaches_results_perchem_RP) %>%
  select(native,AAQ_A2,CLFR_A2,UCQ_A2,LCQ_A2,URP_A2,LRP_A2,ORP_A2,slope,R_squared)
A3_table<-dplyr::left_join(cbind(data.frame(native=all_approaches_native),A3_lm_params),all_approaches_results_perchem) %>%
  dplyr::left_join(all_approaches_results_perchem_RP) %>%
  select(native,AAQ_A3,CLFR_A3,UCQ_A3,LCQ_A3,URP_A3,LRP_A3,ORP_A3,slope,R_squared)
A4_table<-dplyr::left_join(all_approaches_results_perchem,all_approaches_results_perchem_RP) %>%
  dplyr::left_join(med_CD_RF_perchem) %>%
  dplyr::left_join(A4_expert_boot %>% rename(native=Compound,RF_0.025 = med_2.5th_percentile, RF_0.50=med_50th_percentile) %>% select(native,RF_0.025,RF_0.50)) %>%
  select(native,AAQ_A4,CLFR_A4,UCQ_A4,LCQ_A4,URP_A4,LRP_A4,ORP_A4,median_RF,RF_0.025,RF_0.50) %>%
  drop_na()
A5_table<-dplyr::left_join(all_approaches_results_perchem,all_approaches_results_perchem_RP) %>%
  dplyr::left_join(med_CD_RF_perchem) %>%
  dplyr::left_join(A5_global_boot %>% rename(native=Compound,RF_0.025 = med_2.5th_percentile, RF_0.50=med_50th_percentile) %>% select(native,RF_0.025,RF_0.50)) %>%
  select(native,AAQ_A5,CLFR_A5,UCQ_A5,LCQ_A5,URP_A5,LRP_A5,ORP_A5,median_RF,RF_0.025,RF_0.50) %>%
  drop_na()


### COMPARISON OF APPROACHES 1, 4, 5 USING EXAMPLE PREDICTIONS WITH CONFIDENCE INTERVALS FOR EACH CHEMICAL
# Pick one random measurement for each chemical in all_approaches_native
random_per_chem<-data.frame()

set.seed(1)
for(i in 1:length(matched_native)){
  per_chem_rows<-all_approaches_results_long %>% filter(native==matched_native[i]) %>% drop_na()
  random_per_chem<-rbind(random_per_chem,per_chem_rows[sample(nrow(per_chem_rows),size=1),])
}

random_per_chem_plot<-dplyr::left_join(random_per_chem,all_approaches_results) %>% 
  filter(native %in% matched_native) %>%
  filter(native %in% all_approaches_native)

# Make plot
## Figure 5- Plot of random data point + confidence interval per-chemical for A1, A4, A5
ggplot(data=random_per_chem_plot)+
  geom_point(aes(x=reorder(native,upper_A5),y=estimate_A1,color="Approach 1"),position=position_nudge(x=-0.2))+
  geom_linerange(aes(x=reorder(native,upper_A5),ymin=lower_A1,ymax=upper_A1),color="orange",position=position_nudge(x=-0.2))+
  geom_point(aes(x=reorder(native,upper_A5),y=estimate_A4,color="Approach 4"))+
  geom_linerange(aes(x=reorder(native,upper_A5),ymin=lower_A4,ymax=upper_A4),color="blue")+
  geom_point(aes(x=reorder(native,upper_A5),y=estimate_A5,color="Approach 5"),position=position_nudge(x=0.2))+
  geom_linerange(aes(x=reorder(native,upper_A5),ymin=lower_A5,ymax=upper_A5),position=position_nudge(x=0.2),color="black")+
  geom_segment(aes(x=as.numeric(reorder(native,upper_A5))-0.3,xend=as.numeric(reorder(native,upper_A5))+0.3,y=Exp.Amt,yend=Exp.Amt),color="red")+
  scale_y_log10(breaks = scales::trans_breaks("log10", function(y) 10^y),
                labels = scales::trans_format("log10", scales::math_format(10^.x)))+
  theme_bw(base_size = 24)+
  theme(axis.text.x=element_text(angle=90,vjust=0.5,hjust=1),text=element_text(size=24))+
  ylab("Concentration (ng/mL)")+
  xlab("Chemical")+
  scale_color_manual(limits=c("Approach 1","Approach 4","Approach 5","True concentration"),values=c("orange","blue","black","red"),name="Legend")+
  guides(color=guide_legend(override.aes=list(size=3,linetype=c(0,0,0,0))))+
  theme(legend.position = c(0.8, 0.2),
        legend.title = element_blank(),
        legend.background = element_rect(colour = 'black', fill = 'white', linetype='solid'))

### CUMULATIVE DISTRIBUTION FUNCTION PLOTS FOR MODEL ACCURACY QUOTIENT (AQ) AND UPPER CONFIDENCE QUOTIENT (UCQ)
## Figure 4 ##
# For accuracy quotient (AQ)
AAQ_plot_data <- all_approaches_results_long %>%
  group_by(approach) %>%
  mutate(rank = rank(AAQ)/length(AAQ))
  
ggplot(AAQ_plot_data,aes(x = AAQ, y = rank*100, group = approach, color = approach)) +
  #geom_line(size = 1)+
  geom_point(size=1.5)+
  scale_x_log10(breaks = 10^(0:3),
                labels = scales::trans_format("log10", scales::math_format(10^.x)),
                limits = c(10^0, 10^3))+
  scale_y_continuous(n.breaks = 10)+
  scale_color_manual(name="Approach",limits=c("A1","A2","A3","A4","A5"),values=c("orange","purple","green","deepskyblue","black"))+
  theme_bw(base_size = 24)+
  annotation_logticks(sides="b")+
  xlab("Absolute Accuracy Quotient (AAQ)")+
  ylab("Cumulative Percentile")+
  theme(axis.text.x=element_text(size=24,face="bold"),
        axis.text.y=element_text(size=24,face="bold"),
        axis.title.x=element_text(size=24,face="bold"),
        axis.title.y=element_text(size=24,face="bold"),
        #legend.title = element_blank(),
        #legend.position = c(0.85, 0.3),
        #legend.text=element_text(size=24,face="bold"),
        #legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"),
        legend.key.size=unit(0.1,"lines"),
        legend.background = element_blank(),
        legend.box.background = element_rect(fill="white",color="black"),
        plot.margin=margin(0.25,0.25,0.25,0.25,"in")
         )

 # For confidence limit fold range (CLFR)
 CLFR_plot_data <- all_approaches_results_long %>%
   group_by(approach) %>%
   mutate(rank = rank(CLFR)/length(CLFR)) 
 
 ggplot(CLFR_plot_data,aes(x = CLFR, y = rank*100, group = approach, color = approach)) +
   #geom_line(size = 1)+
   geom_point(size=1.5)+
   scale_x_log10(breaks = 10^(0:4),
                 labels = scales::trans_format("log10", scales::math_format(10^.x)),
                 limits = c(10^0,10^4))+
   scale_y_continuous(n.breaks = 10)+
   scale_color_manual(name="Approach",limits=c("A1","A2","A3","A4","A5"),values=c("orange","purple","green","deepskyblue","black"))+
   theme_bw(base_size = 24)+
   annotation_logticks(sides="b")+
   xlab("Confidence Limit Fold Range (CLFR)")+
   ylab("Cumulative Percentile")+
   theme(axis.text.x=element_text(size=24,face="bold"),
         axis.text.y=element_text(size=24,face="bold"),
         axis.title.x=element_text(size=24,face="bold"),
         axis.title.y=element_text(size=24,face="bold"),
         #legend.title = element_text(size=20),
         #legend.title = element_blank(),
         #legend.position = c(0.85, 0.3),
         #legend.text=element_text(size=24,face="bold"),
         #legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"),
         legend.key.size=unit(0.1,"lines"),
         legend.background = element_blank(),
         legend.box.background = element_rect(fill="white",color="black"),
         plot.margin=margin(0.25,0.25,0.25,0.25,"in")
   )

# For Upper Confidence Quotient (UCQ)
UCQ_plot_data <- all_approaches_results_long %>%
  group_by(approach) %>%
  mutate(rank = rank(UCQ)/length(UCQ)) 

ggplot(UCQ_plot_data,aes(x = UCQ, y = rank*100, group = approach, color = approach)) +
  #geom_line(size = 1)+
  geom_point(size=1.5)+
  scale_x_log10(breaks = scales::trans_breaks("log10", function(x) 10^x),
                n.breaks = 5,
                labels = scales::trans_format("log10", scales::math_format(10^.x)))+
  scale_y_continuous(n.breaks = 10)+
  scale_color_manual(name="Approach",limits=c("A1","A2","A3","A4","A5"),values=c("orange","purple","green","deepskyblue","black"))+
  theme_bw(base_size = 24)+
  annotation_logticks(sides="b")+
  xlab(expression(bold('Upper Confidence Quotient (UCQ)')))+
  ylab("Cumulative Percentile")+
  theme(axis.text.x=element_text(size=24,face="bold"),
        axis.text.y=element_text(size=24,face="bold"),
        axis.title.x=element_text(size=24,face="bold"),
        axis.title.y=element_text(size=24,face="bold"),
        #legend.title = element_blank(),
        #legend.position = c(0.85, 0.3),
        #legend.text=element_text(size=24,face="bold"),
        #legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"),
        legend.key.size=unit(0.1,"lines"),
        legend.background = element_blank(),
        legend.box.background = element_rect(fill="white",color="black"),
        plot.margin=margin(0.25,0.25,0.25,0.25,"in")
  )

save(A1_invpred,
     A1_lm_params,
     A2_invpred,
     A2_lm_params,
     A3_invpred,
     A3_lm_params,
     A4_expert_boot_preds,
     A5_global_boot_preds,
     all_approaches_results,
     all_approaches_results_long,
     all_approaches_results_perchem,
     all_approaches_results_perchem_RP,
     all_approaches_results_SI,
     file="Pu_et_al_qNTA_metrics_results.RData")
