# Created 02/09/2023
# Author: Shirley Pu
# Code used for the paper "Establishing performance metrics for quantitative non-targeted analysis: a demonstration using per- and polyfluoroalkyl substances"
# Contains code used for data analysis that is included in Supporting Information (Supplementary File 3)

# Load libraries
library(tidyverse)
library(investr)
library(ggridges)
library(lme4)
library(lmerTest)

# Source custom functions
source("Pu_et_al_qNTA_metrics_Supplementary_File_2_scripts.R")

# load instrument measurements, processed in Pu_et_al_qNTA_metricst_data.R
load("Pu_et_al_qNTA_metrics_data.RData")

# Load data results from main manuscript data analysis
load("Pu_et_al_qNTA_metrics_results.RData")

# Create vectors containing native and IS compounds
# They are in order of increasing retention time
# matched_native and matched_IS are used for Approach 1
matched_native<-c("PFBA","PFPeA","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","6:2 FTS","PFOA","PFHxS","PFNA","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFDoDA","PFOSA","PFTeDA")
matched_IS<-c("MPFBA","M5PFPeA","M2-4_2_FTS","M3PFBS","M5PFHxA","M3HFPO-DA","M4PFHpA","M2-6_2_FTS","M8PFOA","M3PFHxS","M9PFNA","M2-8_2_FTS","M6PFDA","d3-N-MeFOSAA","M8PFOS","d5-N-EtFOSAA","M7PFUnDA","M2PFDoDA","M8PFOSA","M2PFTeDA_-13CO2")
# all_targeted_native contains all 29 native chemicals included in the study
all_targeted_native<-c("PFMOAA","PFBA","PFECA-F","PFPeA","PFECA-A","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","PFPeS","6:2 FTS","PFOA","PFHxS","NBP2","PFNA","PFHpS","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFNS","PFDoDA","PFDS","PFTrDA","PFOSA","PFTeDA")
# all_approaches_native chemicals are those with measurements available for both targeted acquisition (XCalibur Quan) and non-targeted acquisition (Compound Discoverer). n=26 (no PFMOAA, PFBA, PFECA-F because below signal threshold. PFTeDA_-CO2 excluded due to bad calibration curve)
all_approaches_native<-c("PFPeA","PFECA-A","4:2 FTS","PFBS","PFHxA","HFPODA","PFHpA","PFPeS","6:2 FTS","PFOA","PFHxS","NBP2","PFNA","PFHpS","8:2 FTS","PFDA","N-MeFOSAA","PFOS","N-EtFOSAA","PFUnDA","PFNS","PFDoDA","PFDS","PFTrDA","PFOSA","PFTeDA")

## Supporting Information 8.0 ##
## Pairwise comparisons of AAQ, CLFR, and UCQ metrics using Wilcoxon signed-rank tests
approach_combns<-combn(x=c("A1","A2","A3","A4","A5"),m=2) %>% t()

# AAQ
approaches_AAQ_median<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with("AAQ")) %>%
  group_by(native) %>%
  summarize(across(contains("AAQ_"), ~median(.x,na.rm=TRUE)))
approaches_AAQ_wilcox_median<-vector(mode="list",length=nrow(approach_combns))
for(i in 1:(nrow(approach_combns))){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  AAQ_1<-paste0("AAQ_",pair1)
  AAQ_2<-paste0("AAQ_",pair2)
  AAQ_diff<-(approaches_AAQ_median %>% pull(AAQ_2) %>% log10())-(approaches_AAQ_median %>% pull(AAQ_1) %>% log10())
  AAQ_diff_name<-paste0("log_AAQ_diff_",pair2,"_",pair1)
  approaches_AAQ_median<-approaches_AAQ_median %>% 
    mutate(!!AAQ_diff_name := AAQ_diff)
  approach_AAQ_wilcox_median<-wilcox.test(approaches_AAQ_median %>% pull(AAQ_diff_name) %>% .[!is.na(.)],conf.int=TRUE)
  AAQ_diff_length<-length(AAQ_diff[!is.na(AAQ_diff)])
  # the estimate is the pseudomedian
  # get fold-difference from pseudomedian
  AAQ_wilcox_median_FD<-10^approach_AAQ_wilcox_median$estimate
  approaches_AAQ_wilcox_median[[i]]<-c(approach_AAQ_wilcox_median,median_FD=AAQ_wilcox_median_FD,n=AAQ_diff_length)
  names(approaches_AAQ_wilcox_median)[[i]]<-paste0(pair2,"_",pair1)
}

# Comparing difference in CLFR  between Approach 1, 2, 3, 4, 5 (pairs in order) using Wilcoxon rank-sum tests
approaches_CLFR_median<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with("CLFR")) %>%
  group_by(native) %>%
  summarize(across(contains("CLFR_"), ~median(.x,na.rm=TRUE)))
approaches_CLFR_wilcox_median<-vector(mode="list",length=nrow(approach_combns))
for(i in 1:(nrow(approach_combns))){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  CLFR_1<-paste0("CLFR_",pair1)
  CLFR_2<-paste0("CLFR_",pair2)
  CLFR_diff<-(approaches_CLFR_median %>% pull(CLFR_2) %>% log10())-(approaches_CLFR_median %>% pull(CLFR_1) %>% log10())
  CLFR_diff_name<-paste0("log_CLFR_diff_",pair2,"_",pair1)
  approaches_CLFR_median<-approaches_CLFR_median %>% 
    mutate(!!CLFR_diff_name := CLFR_diff)
  CLFR_diff_length<-length(CLFR_diff[!is.na(CLFR_diff)])
  approach_CLFR_wilcox_median<-wilcox.test(approaches_CLFR_median %>% pull(CLFR_diff_name) %>% .[!is.na(.)],conf.int=TRUE)
  # the estimate is the pseudomedian
  # get fold-difference from pseudomedian
  CLFR_wilcox_median_FD<-10^approach_CLFR_wilcox_median$estimate
  approaches_CLFR_wilcox_median[[i]]<-c(approach_CLFR_wilcox_median,median_FD=CLFR_wilcox_median_FD,n=CLFR_diff_length)
  names(approaches_CLFR_wilcox_median)[[i]]<-paste0(pair2,"_",pair1)
}

# UCQ
approaches_UCQ_median<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with("UCQ")) %>%
  group_by(native) %>%
  summarize(across(contains("UCQ"), ~median(.x,na.rm=TRUE))) %>%
  mutate(across(contains("UCQ"), ~replace(., . < 1, NA)))
approaches_UCQ_wilcox_median<-vector(mode="list",length=nrow(approach_combns))
for(i in 1:(nrow(approach_combns))){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  UCQ_1<-paste0("UCQ_",pair1)
  UCQ_2<-paste0("UCQ_",pair2)
  UCQ_diff<-(approaches_UCQ_median %>% pull(UCQ_2) %>% log10())-(approaches_UCQ_median %>% pull(UCQ_1) %>% log10())
  UCQ_diff_name<-paste0("log_UCQ_diff_",pair2,"_",pair1)
  approaches_UCQ_median<-approaches_UCQ_median %>% 
    mutate(!!UCQ_diff_name := UCQ_diff)
  UCQ_diff_length<-length(UCQ_diff[!is.na(UCQ_diff)])
  approach_UCQ_wilcox_median<-wilcox.test(approaches_UCQ_median %>% pull(UCQ_diff_name) %>% .[!is.na(.)],conf.int=TRUE)
  # the estimate is the pseudomedian
  # get fold-difference from pseudomedian
  UCQ_wilcox_median_FD<-10^approach_UCQ_wilcox_median$estimate
  approaches_UCQ_wilcox_median[[i]]<-c(approach_UCQ_wilcox_median,median_FD=UCQ_wilcox_median_FD,n=UCQ_diff_length)
  names(approaches_UCQ_wilcox_median)[[i]]<-paste0(pair2,"_",pair1)
}

# Wilcoxon ranked-sum for LCQ (all comparisons)
approaches_LCQ_median<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with("LCQ")) %>%
  group_by(native) %>%
  summarize(across(contains("LCQ"), ~median(.x,na.rm=TRUE))) %>%
  mutate(across(contains("LCQ"), ~replace(., . < 1, NA)))
approaches_LCQ_wilcox_median<-vector(mode="list",length=nrow(approach_combns))
for(i in 1:(nrow(approach_combns))){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  LCQ_1<-paste0("LCQ_",pair1)
  LCQ_2<-paste0("LCQ_",pair2)
  LCQ_diff<-(approaches_LCQ_median %>% pull(LCQ_2) %>% log10())-(approaches_LCQ_median %>% pull(LCQ_1) %>% log10())
  LCQ_diff_name<-paste0("log_LCQ_diff_",pair2,"_",pair1)
  approaches_LCQ_median<-approaches_LCQ_median %>% 
    mutate(!!LCQ_diff_name := LCQ_diff)
  LCQ_diff_length<-length(LCQ_diff[!is.na(LCQ_diff)])
  approach_LCQ_wilcox_median<-wilcox.test(approaches_LCQ_median %>% pull(LCQ_diff_name) %>% .[!is.na(.)],conf.int=TRUE)
  # the estimate is the pseudomedian
  # get fold-difference from pseudomedian
  LCQ_wilcox_median_FD<-10^approach_LCQ_wilcox_median$estimate
  approaches_LCQ_wilcox_median[[i]]<-c(approach_LCQ_wilcox_median,median_FD=LCQ_wilcox_median_FD,n=LCQ_diff_length)
  names(approaches_LCQ_wilcox_median)[[i]]<-paste0(pair2,"_",pair1)
}

## SI 9.0: SECONDARY USE CASE ## 
# Statistical comparison of LCQ for Approaches 1-5
# Comparing difference in lower confidence quotients (LCQ) between Approaches 1, 2, 3, 4, 5 (pairs in order) using linear mixed-model

# LCQ (Lower Confidence Quotient)
approaches_LCQ<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with('LCQ_')) %>%
  mutate(across(contains("LCQ_"), ~replace(., . < 1, NA)))
approaches_LCQ_lmer<-vector(mode="list",length=nrow(approach_combns)-1)
LCQ_diff_lengths<-vector(mode="numeric",length=nrow(approach_combns)-1)
for(i in 1:(nrow(approach_combns)-1)){
  pair1<-approach_combns[i,1]
  pair2<-approach_combns[i,2]
  LCQ_1<-paste0("LCQ_",pair1)
  LCQ_2<-paste0("LCQ_",pair2)
  LCQ_diff<-(approaches_LCQ %>% pull(LCQ_2) %>% log10())-(approaches_LCQ %>% pull(LCQ_1) %>% log10())
  LCQ_diff_name<-paste0("log_LCQ_",pair1,"_",pair2)
  approaches_LCQ<-approaches_LCQ %>% 
    mutate(!!LCQ_diff_name := LCQ_diff)
  LCQ_diff_lengths[i]<-approaches_LCQ %>% filter(!!as.symbol(LCQ_1)>=1 & !!as.symbol(LCQ_2)>=1) %>% filter(!is.na(!!as.symbol(LCQ_diff_name))) %>% nrow()
  curr_lmer<-lmer(paste0("`",LCQ_diff_name,"`","~(1|native)"), data=approaches_LCQ %>% filter(.data[[LCQ_1]]>=1 & .data[[LCQ_2]]>=1) %>% filter(!is.na(.data[[LCQ_diff_name]])))
  approaches_LCQ_lmer[[i]]<-curr_lmer
  names(approaches_LCQ_lmer)[[i]]<-paste0(pair2,"_",pair1)
}

approach_LCQ_lmer_coef<-sapply(approaches_LCQ_lmer,function(x) summary(x)$coefficients) %>% t()
colnames(approach_LCQ_lmer_coef)<-c("Estimate","Std.Error","df","t_value","p_value")
approach_LCQ_lmer_coef<-as.data.frame(approach_LCQ_lmer_coef) %>% 
  mutate(GM_FD=10^Estimate) %>%
  mutate(n=LCQ_diff_lengths)

# Wilcoxon (A4-A5 only)
approaches_LCQ_A4_A5<-all_approaches_results %>% 
  select(native,Exp.Amt,replicate,starts_with('LCQ_')) %>%
  group_by(native) %>% 
  summarize_at(c("LCQ_A4","LCQ_A5"),median,na.rm=TRUE) %>% 
  filter(LCQ_A4>=1 & LCQ_A5>=1) %>% 
  mutate(log_LCQ_diff_A4_A5=log10(LCQ_A5)-log10(LCQ_A4))

approaches_LCQ_wilcox<-wilcox.test(approaches_LCQ_A4_A5 %>% pull("log_LCQ_diff_A4_A5"),conf.int=TRUE)
# the estimate is the pseudomedian
# we will try using the mean of the difference in log10(LCQ) as the estimate
approaches_LCQ_wilcox<-c(approaches_LCQ_wilcox,
                         median_FD=10^(approaches_LCQ_wilcox$estimate))

# Median Lower Confidence Quotient
median(A1_invpred$LCQ)
median(A2_invpred$LCQ)
median(A3_invpred$LCQ)
median(A4_expert_boot_preds$LCQ)
median(A5_global_boot_preds$LCQ)

# Ninety-fifth percentile LCQ
quantile(A1_invpred$LCQ,0.95)
quantile(A2_invpred$LCQ,0.95)
quantile(A3_invpred$LCQ,0.95)
quantile(A4_expert_boot_preds$LCQ,0.95)
quantile(A5_global_boot_preds$LCQ,0.95)

# LOWER RELIABILITY PERCENTAGE (LRP)
# Per-approach
sum(A1_invpred$lower<=A1_invpred$Exp.Amt)/nrow(A1_invpred)
sum(A2_invpred$lower<=A2_invpred$Exp.Amt)/nrow(A2_invpred)
sum(A3_invpred$lower<=A3_invpred$Exp.Amt)/nrow(A3_invpred)
sum(A4_expert_boot_preds$lower<=A4_expert_boot_preds$Exp.Amt)/nrow(A4_expert_boot_preds)
sum(A5_global_boot_preds$lower<=A5_global_boot_preds$Exp.Amt)/nrow(A5_global_boot_preds)

### CALIBRATION CURVES FOR A1-A3, SLOPE AND RF DISTRIBUTIONS ###
## Figure S1 ##
A1_lm_params$approach<-"A1"
A2_lm_params$approach<-"A2"
A3_lm_params$approach<-"A3"
A1_A3_lm_params<-rbind(A1_lm_params,A2_lm_params,A3_lm_params)

# Histograms for Slope
ggplot(A1_A3_lm_params, aes(slope))+
  geom_histogram()+
  theme_bw(base_size=24)+
  scale_y_continuous(breaks=c(2,4,6,8,10,12,14,16,18),limits=c(0,14),expand=c(0,0))+
  xlab("Slope")+
  ylab("Count")+
  facet_grid(cols=vars(approach))+
  theme(panel.spacing.x=unit(15,"pt"))

# Histograms for R-squared
ggplot(A1_A3_lm_params, aes(R_squared))+
  geom_histogram()+
  xlab(expression(R^2))+
  theme_bw(base_size=24)+
  scale_y_continuous(breaks=c(2,4,6,8,10,12,14,16,18),limits=c(0,14),expand=c(0,0))+
  ylab("Count")+
  facet_grid(cols=vars(approach))+
  theme(panel.spacing.x=unit(28,"pt"))

### RESPONSE FACTOR DISTRIBUTION FOR TARGETED ACQUISITION DATA
## Figure S2 ##
XQ_RFs<-data.frame()
for(i in 1:length(all_targeted_native)){
  XQ_RFs<-rbind(XQ_RFs,get_RF(all_targeted_native[i],all_targeted_native[i],XQuan_wide,ratio=FALSE))
}
XQ_RFs<-XQ_RFs[complete.cases(XQ_RFs),]
XQ_RFs<-dplyr::left_join(XQ_RFs,PFAS_solvent_only_RT,by=c("native"="Compound"))

ggplot(XQ_RFs,aes(x=RF,y=reorder(native,-RT)))+
  xlab(expression('Response Factor (ng/mL)'^-1))+
  ylab("Chemical")+
  geom_density_ridges(scale=2,alpha=.5,bandwidth=0.1)+
  scale_x_log10(breaks = scales::trans_breaks("log10", function(x) 10^x),
                labels = scales::trans_format("log10", scales::math_format(10^.x)))+
  theme_bw(base_size=22)

### ACCURACY QUOTIENT FOR A1-A5 ###
## Figure S3 ##
# Plot AQ distributions for each approach 
ggplot(all_approaches_results_long,aes(x=approach,y=AQ))+
  geom_boxplot(outlier.shape=NA)+ #avoid plotting outliers twice
  geom_jitter(position=position_jitter(width=.2,height=0))+
  scale_y_log10(breaks = scales::trans_breaks("log10", function(y) 10^y),
                labels = scales::trans_format("log10", scales::math_format(10^.x)),
                limits = c(0.001,1000))+
  ylab("Accuracy Quotient (AQ)")+
  xlab("Approach")+
  theme_bw(base_size=24)

# AQ median
median(all_approaches_results %>% filter(!is.na(AQ_A1)) %>% pull('AQ_A1'))
median(all_approaches_results %>% filter(!is.na(AQ_A2)) %>% pull('AQ_A2'))
median(all_approaches_results %>% filter(!is.na(AQ_A3)) %>% pull('AQ_A3'))
median(all_approaches_results %>% filter(!is.na(AQ_A4)) %>% pull('AQ_A4'))
median(all_approaches_results %>% filter(!is.na(AQ_A5)) %>% pull('AQ_A5'))

### HEATMAPS OF PER-CHEMICAL METRICS ###
## Figure S4 ##
# AAQ
# ggplot geom_tile heatmap
AAQ_perchem_ggtile<-all_approaches_results_perchem %>%
  select(contains(c("native","AAQ","RT"))) %>%
  pivot_longer(cols=contains("AAQ"),
               names_to="approach",
               names_prefix="AAQ_",
               values_to="AAQ")
ggplot(AAQ_perchem_ggtile,aes(approach,reorder(native,-RT)))+
  geom_tile(aes(fill=log10(AAQ)),color="gray85")+
  scale_fill_gradient(low="gray85",high="black",na.value="white")+
  xlab("Approach")+
  ylab("Chemical")+
  theme_bw(base_size=20)+
  theme(axis.text.x=element_text(angle=90,vjust=0.5,hjust=1),
        legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm")
  )+
  labs(tag="a")

# CLFR
CLFR_perchem_ggtile<-all_approaches_results_perchem %>%
  select(contains(c("native","CLFR","RT"))) %>%
  pivot_longer(cols=contains("CLFR"),
               names_to="approach",
               names_prefix="CLFR_",
               values_to="CLFR")
ggplot(CLFR_perchem_ggtile,aes(approach,reorder(native,-RT)))+
  geom_tile(aes(fill=log10(CLFR)),color="gray85")+
  scale_fill_gradient(low="gray85",high="black",na.value="white")+
  xlab("Approach")+
  ylab("Chemical")+
  theme_bw(base_size=20)+
  theme(axis.text.x=element_text(angle=90,vjust=0.5,hjust=1),
        legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"))+
  labs(tag="b")

# Upper Confidence Quotient (UCQ)
UCQ_perchem_ggtile<-all_approaches_results_perchem %>%
  select(contains(c("native","UCQ","RT"))) %>%
  pivot_longer(cols=contains("UCQ"),
               names_to="approach",
               names_prefix="UCQ_",
               values_to="UCQ")
ggplot(UCQ_perchem_ggtile,aes(approach,reorder(native,-RT)))+
  geom_tile(aes(fill=log10(UCQ)),color="gray85")+
  scale_fill_gradient(low="gray85",high="black",na.value="white")+
  xlab("Approach")+
  ylab("Chemical")+
  theme_bw(base_size=20)+
  theme(axis.text.x=element_text(angle=90,vjust=0.5,hjust=1),
        legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"))+
  labs(tag="c")

# Overall Reliability Percentage (ORP)
ORP_perchem_ggtile<-all_approaches_results_perchem_RP %>%
  select(contains(c("native","ORP","RT"))) %>%
  pivot_longer(cols=contains("ORP"),
               names_to="approach",
               names_prefix="ORP_",
               values_to="ORP")
ggplot(ORP_perchem_ggtile,aes(x=approach,y=reorder(native,-RT)))+
  geom_tile(aes(fill=ORP),color="gray85")+
  scale_fill_gradient(low="gray85",high="black",na.value="white")+
  xlab("Approach")+
  ylab("Chemical")+
  theme_bw(base_size=20)+
  theme(axis.text.x=element_text(angle=90,vjust=0.5,hjust=1),
        legend.margin=margin(t=0,r=0,b=0,l=0,unit="mm"))+
  labs(tag="d")
