# Author: Shirley Pu
# Functions used for the paper "Establishing performance metrics for quantitative non-targeted analysis: a demonstration using per- and polyfluoroalkyl substances"

### FUNCTIONS FOR CALIBRATION CURVES AND INVERSE ESTIMATION ### 

# Function which plots the calibration curve using ggplot() 
# @param native_Cmp: String that is the name of the native compound 
# @param IS_Cmp: String that is the name of the internal standard 
# @param data_in: Tibble with one compound per row; areas for a given sample are columns 
# @param ratio: If TRUE, use area ratio instead of native area only as input to the model
# @param plot_out: If TRUE, return the ggplot. If FALSE, return the lm object 
# @param save: If TRUE, save the image as a .png file
ggplot_calcurve<-function(native_Cmp,IS_Cmp,data_in,ratio=TRUE,plot_out=TRUE,save=FALSE){
  # Filter for standards that have detected areas
  data_noNA <- data_in %>% filter(INPUT %in% c(native_Cmp,IS_Cmp)) %>% select(where(~!any(is.na(.))))
  native_data<- data_noNA %>% filter(INPUT==native_Cmp)
  IS_data<- data_noNA %>% filter(INPUT==IS_Cmp) 
  # Get expected amounts from names; check number of unique calibration amounts
  cal_levels<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% unique() 
  cal_amounts<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% gsub("Cal","",.) %>% gsub("-",".",.) %>% unlist() %>% as.numeric()
  # Function for plotting confidence interval, also adds equation to graph 
  # @param lm_in: The lm() object for the model 
  # @param input_col: The df column that is input for the lm() model 
  add_text<-function(lm_in){
    # Add equation at top. Check if formula has ^2 (quadratic) or 'Ratio' 
    cf<-round(coef(lm_in),2)
    if(length(grep("Ratio",as.character(lm_in$call[2])))>0){
      eq<-paste0("Log\u2082","(Area Ratio) = ",formatC(cf[1],digits=2),ifelse(sign(cf[2])==1,"+","-"),formatC(abs(cf[2]),digits=2),"Log\u2082","(Experimental Amount)")
    }
    else{
      eq<-paste0("Log\u2082","(Native Area) = ",formatC(cf[1],digits=2),ifelse(sign(cf[2])==1,"+","-"),formatC(abs(cf[2]),digits=2),"Log\u2082","(Experimental Amount)")
    }
    return(eq)
  }
  # Plotting for different types of models
  if(ratio){
    response_ratios<-(native_data %>% select(starts_with('Cal')))/IS_data %>% select(starts_with('Cal')) 
    df<-data.frame(Exp.Amt=cal_amounts,log_Exp.Amt=log2(cal_amounts),log_Response.Ratio=log2(as.numeric(response_ratios)))
    df<-df[complete.cases(df),]
    stored_lm<-lm(df,formula=log_Response.Ratio~log_Exp.Amt)
    stored_predict<-predict(stored_lm,interval="prediction")
    ix<-sort(df$log_Exp.Amt,index.return=T)$ix
    plotting_df<-data.frame(a=c(rev(df$log_Exp.Amt[ix]),df$log_Exp.Amt[ix]),b=c(rev(stored_predict[ix,3]),stored_predict[ix,2]))
    plt<-ggplot(df,aes(x=log_Exp.Amt,y=log_Response.Ratio))+geom_point()+ggtitle(paste(native_Cmp,IS_Cmp,"Calibration Curve"))+xlab(expression('Log'[2]*'(Experimental Amount) (ng/mL)'))+ylab(expression('Log'[2]*'(Area Ratio)'))+geom_line(data=data.frame(x=df$log_Exp.Amt[ix],y=stored_predict[ix,1]),aes(x=x,y=y))+geom_polygon(data=plotting_df,aes(x=a,y=b),alpha=0.2)+labs(subtitle=paste0(add_text(stored_lm),"\n ","R\u00b2",": ",formatC(summary(stored_lm)$r.squared),digits=2))+theme_classic()+theme(aspect.ratio=1)
    # fix names for export file if needed
    native_Cmp<-gsub(":2 ","_2_",native_Cmp)
    IS_Cmp<-gsub(":2 ","_2_",IS_Cmp)
    if(save){
      ggsave(paste0(native_Cmp,"_",IS_Cmp,"_LL_ratio_calibration_curve.png"),width=2100,height=2100,units="px")
    }
    if(plot_out){
      return(plt)        
    }
  }
  else{
    df<-data.frame(Exp.Amt=cal_amounts,Area=native_data %>% select(starts_with('Cal')) %>% unlist() %>% as.numeric())
    df$log_Exp.Amt<-log2(df$Exp.Amt)
    df$log_Area<-log2(df$Area)
    df<-df[complete.cases(df),]
    stored_lm<-lm(df,formula=log_Area~log_Exp.Amt)
    stored_predict<-predict(stored_lm,interval="prediction")
    ix<-sort(df$log_Exp.Amt,index.return=T)$ix
    plotting_df<-data.frame(a=c(rev(df$log_Exp.Amt[ix]),df$log_Exp.Amt[ix]),b=c(rev(stored_predict[ix,3]),stored_predict[ix,2]))
    plt<-ggplot(df,aes(x=log_Exp.Amt,y=log_Area))+geom_point()+ggtitle(paste(native_Cmp,"Calibration Curve"))+xlab(expression('Log'[2]*'(Experimental Amount) (ng/mL)'))+ylab(expression('Log'[2]*'(Native Area)'))+geom_line(data=data.frame(x=df$log_Exp.Amt[ix],y=stored_predict[ix,1]),aes(x=x,y=y))+geom_polygon(data=plotting_df,aes(x=a,y=b),alpha=0.2)+labs(subtitle=paste0(add_text(stored_lm),"\n ","R\u00b2",": ",formatC(summary(stored_lm)$r.squared),digits=2))+theme_classic()+theme(aspect.ratio=1)
    # fix names for export file if needed
    native_Cmp<-gsub(":2 ","_2_",native_Cmp)
    if(save){
      ggsave(paste0(native_Cmp,"_LL_area_calibration_curve.png"),width=2100,height=2100,units="px")
    }
    if(plot_out){
      return(plt) 
    }
  }
  return(stored_lm)
}

# Function which gets the inverse estimate for a native compound based on its calibration curve with an internal standard
# @param native_Cmp: String that is the name of the native compound. 
# @param IS_Cmp: String that is the name of the internal standard. 
# @param data_in: Tibble with one compound per row; the areas for a sample/CD input file are all in one column
# @param pred_pts: String or list of strings of the column names to use for inverse prediction
# @param loglog: If TRUE, use log-log fit instead of quadratic fit with weighted errors 
# @param ratio: If TRUE, use area ratio instead of native area as input to model
# @param as_df: Output is a dataframe instead of a table. Default is TRUE
get_invest<-function(native_Cmp,IS_Cmp,data_in,pred_pts=NA,ratio=TRUE,as_df=TRUE){
  # Filter for standards that have detected areas
  data_noNA <- data_in %>% filter(INPUT %in% c(native_Cmp,IS_Cmp)) %>% select(where(~!any(is.na(.))))
  native_data<- data_noNA %>% filter(INPUT==native_Cmp)
  IS_data<-data_noNA %>% filter(INPUT==IS_Cmp) 
  # Get expected amounts from names
  cal_levels<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% unique()
  cal_amounts<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% gsub("Cal","",.) %>% gsub("-",".",.) %>% as.numeric()
  # Calculate response ratios
  response_ratios<-(native_data %>% select(starts_with('Cal')))/(IS_data %>% select(starts_with('Cal'))) 
  if(ratio){
    df<-data.frame(Exp.Amt=cal_amounts,log_Exp.Amt=log2(cal_amounts),log_Response.Ratio=as.numeric(response_ratios) %>% log2())
    df<-df[complete.cases(df),]
    plot(df$log_Exp.Amt,df$log_Response.Ratio,main=paste(native_Cmp,IS_Cmp,"Cal Curve"),xlab="Log2 Experimental Amount",ylab="Log2 Area Ratio")
    cal_curve<-lm(formula=log_Response.Ratio~log_Exp.Amt,df)  
    curve(predict(cal_curve,data.frame(log_Exp.Amt=x)),add=T)
  }
  else{
    df<-data.frame(Exp.Amt=cal_amounts,Area=native_data %>% select(starts_with('Cal')) %>% as.numeric())
    df$log_Exp.Amt<-log2(df$Exp.Amt)
    df$log_Area<-log2(df$Area)
    df<-df[complete.cases(df),]
    plot(df$log_Exp.Amt, df$log_Area,main=paste(native_Cmp,"Cal Curve"),xlab="Log2 Experimental Amount",ylab="Log2 Native Area")
    cal_curve<-lm(formula=log_Area~log_Exp.Amt,df)  
    curve(predict(cal_curve,data.frame(log_Exp.Amt=x)),add=T)        
  }
  preds<-list()
  if(all(!is.na(pred_pts))){
    if(ratio){
      response_pts<-(native_data %>% select(pred_pts))/(IS_data %>% select(pred_pts))
      response_pts<-log2(as.numeric(response_pts))
    }
    else{
      response_pts<-native_data %>% select(pred_pts) %>% as.numeric() %>% log2() 
    }
    for(s in 1:length(response_pts)){
      preds[[s]]<-calibrate(cal_curve,y0=response_pts[s])
    }
  }
  if(loglog){
    if(ratio){
      for(s in 1:length(df$log_Response.Ratio)){
        preds[[s]]<-calibrate(cal_curve,y0=df$log_Response.Ratio[s])
      }
    }
    else{
      for(s in 1:length(df$log_Area)){
        preds[[s]]<-calibrate(cal_curve,y0=df$log_Area[s])
      } 
    }
  }
  
  if(as_df==TRUE){
    if(all(!is.na(pred_pts))){
      # Note that if there are spike replicates, that part needs to be removed (last 3 characters)
      exp_amts<-pred_pts %>% str_extract(.,"[[:digit:]]+") %>% as.numeric()
    }
    else{
      exp_amts<-df$Exp.Amt
    }
    df_out<-data.frame(native_Cmp=rep(native_Cmp,length(preds)),IS_Cmp=rep(IS_Cmp,length(preds)),Exp.Amt=exp_amts)
    df_out<-cbind(df_out,data.frame(estimate=unlist(map(preds,1)),lower=unlist(map(preds,2)),upper=unlist(map(preds,3))))
    df_out$accuracy_quotient<-10^(abs(log10((2^df_out$estimate))/log10(exp_amts)))
    if(ratio){
      df_out$log_Response.Ratio<-df$log_Response.Ratio
    }
    else{
      if(all(!is.na(pred_pts))){
        df_out$log_Area<-log2(native_data %>% select(pred_pts) %>% as.numeric())
      }
      else{
        df_out$log_Area<-df$log_Area
      } 
    }
    return(df_out)
  }
  return(preds) 
}

# Function which gets the inverse estimate for a native compound based on its calibration curve with an internal standard
# Based on get_invest(), but instead of inputting pred_pts, one calibration point is excluded and used as the spike which concentration is estimated for
# Repeats in a loop until all calibration points (except those at the lowest and highest calibration amounts) have been used as spikes
# @param native_Cmp: String that is the name of the native compound. 
# @param IS_Cmp: String that is the name of the internal standard. 
# @param data_in: Tibble with one compound per row; the areas for a sample/CD input file are all in one column
# @param loglog: If TRUE, use log-log fit instead of quadratic fit with weighted errors 
# @param ratio: If TRUE, use area ratio instead of native area as input to model
# @param as_df: Output is a dataframe instead of a table. Default is TRUE
get_invest_leaveout<-function(native_Cmp,IS_Cmp,data_in,ratio=TRUE,as_df=TRUE){
  # Filter for standards that have detected areas (need to find a way to exclude NA)
  data_noNA <- data_in %>% filter(INPUT %in% c(native_Cmp,IS_Cmp)) %>% select(where(~!any(is.na(.))))
  native_data<- data_noNA %>% filter(INPUT==native_Cmp)
  IS_data<-data_noNA %>% filter(INPUT==IS_Cmp)
  # Get expected amounts from names
  cal_levels<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% unique()
  preds<-list()
  rep_list<-vector(mode="numeric")
  areas_list<-vector(mode="numeric")
  slopes_list<-vector(mode="numeric")
  Rsquared_list<-vector(mode="numeric")
  for(i in 1:length(native_data %>% select(starts_with('Cal')) %>% colnames())){
    # save column name of excluded calibration point
    cal_excluded<-native_data %>% select(starts_with('Cal')) %>% colnames() %>% .[i]
    rep_list<-append(rep_list,str_sub(cal_excluded,start=-1) %>% as.numeric())
    areas_list<-append(areas_list,native_data %>% select(cal_excluded) %>% as.numeric())
    pred_pts<-cal_excluded
    cal_amounts<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% select(-cal_excluded) %>% colnames(), end=-3) %>% gsub("Cal","",.) %>% gsub("-",".",.) %>% as.numeric()
    # Calculate response ratios
    response_ratios<-(native_data %>% select(starts_with('Cal')) %>% select(-cal_excluded) %>% as.numeric())/(IS_data %>% select(starts_with('Cal')) %>% select(-cal_excluded) %>% as.numeric()) 
    if(ratio){
      df<-data.frame(Exp.Amt=cal_amounts,log_Exp.Amt=log2(cal_amounts),log_Response.Ratio=as.numeric(response_ratios) %>% log2())
      df<-df[complete.cases(df),]
      plot(df$log_Exp.Amt,df$log_Response.Ratio,main=paste(native_Cmp,IS_Cmp,"Cal Curve"),xlab="Log2 Experimental Amount",ylab="Log2 Area Ratio")
      cal_curve<-lm(formula=log_Response.Ratio~log_Exp.Amt,df)  
      curve(predict(cal_curve,data.frame(log_Exp.Amt=x)),add=T)
    }
    else{
      df<-data.frame(Exp.Amt=cal_amounts,Area=native_data %>% select(starts_with('Cal')) %>% select(-cal_excluded) %>% as.numeric())
      df$log_Exp.Amt<-log2(df$Exp.Amt)
      df$log_Area<-log2(df$Area)
      df<-df[complete.cases(df),]
      plot(df$log_Exp.Amt, df$log_Area,main=paste(native_Cmp,"Cal Curve"),xlab="Log2 Experimental Amount",ylab="Log2 Native Area")
      cal_curve<-lm(formula=log_Area~log_Exp.Amt,df)  
      curve(predict(cal_curve,data.frame(log_Exp.Amt=x)),add=T)        
    }
    # add current slope and current Rsquared
    slopes_list<-append(slopes_list,cal_curve$coefficients[2])
    Rsquared_list<-append(Rsquared_list,summary(cal_curve)$r.squared)
    
    if(ratio){
      response_pts<-(native_data %>% select(pred_pts))/(IS_data %>% select(pred_pts))
      response_pts<-log2(as.numeric(response_pts))
    }
    else{
      response_pts<-native_data %>% select(pred_pts) %>% as.numeric() %>% log2() 
    }
    preds[[i]]<-calibrate(cal_curve,y0=response_pts)
  }
  if(as_df==TRUE){
    exp_amts<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% gsub("Cal","",.) %>% gsub("-",".",.) %>% as.numeric()
    df_out<-data.frame(native_Cmp=rep(native_Cmp,length(preds)),IS_Cmp=rep(IS_Cmp,length(preds)),Exp.Amt=exp_amts,replicate=rep_list,Area=areas_list)
    df_out<-cbind(df_out,data.frame(estimate=unlist(map(preds,1)),lower=unlist(map(preds,2)),upper=unlist(map(preds,3))))
    estimate<-2^df_out$estimate
    lower<-2^df_out$lower
    upper<-2^df_out$upper
    log_fold_diff<-log10(estimate/exp_amts)
    #df_out$in_interval<-unlist(lapply(1:length(exp_amts),function(x) between(exp_amts[x],lower[x],upper[x])))
    df_out$AQ<-10^(log_fold_diff)
    df_out$AAQ<-10^abs(log_fold_diff)                        
    df_out$CLFR<-upper/lower
    df_out$LCQ<-exp_amts/lower
    df_out$UCQ<-upper/exp_amts
    df_out$logspace_estimate<-df_out$estimate
    df_out$logspace_lower<-df_out$lower
    df_out$logspace_upper<-df_out$upper
    df_out$slope<-slopes_list
    df_out$Rsquared<-Rsquared_list
    #replace estimate, lower, upper with rectilinear space estimates
    df_out$estimate<-estimate
    df_out$lower<-lower
    df_out$upper<-upper
    
    return(df_out)
  }
  return(preds) 
}

### FUNCTIONS FOR CALCULATING NEW VARIABLES ### 

# Functions which calculates response factor or relative response factor
# @param native_Cmp: The native chemical
# @param IS_Cmp: The IS chemical, only used when ratio=TRUE
# @param data_in: The input tibble or dataframe 
# @param loglog: If TRUE, use log-log transformation of the data 
# @param ratio: If TRUE, use area ratio model (relative response factor) instead of native area model (response factor)
# @return A dataframe containing the chemical names, experimental amounts, and response factors or response ratios 
get_RF<-function(native_Cmp,IS_Cmp,data_in,ratio=TRUE){
  # Filter for standards that have detected areas 
  data_noNA <- data_in %>% filter(INPUT %in% c(native_Cmp,IS_Cmp)) %>% select(where(~!any(is.na(.))))
  native_data<- data_noNA %>% filter(INPUT==native_Cmp) 
  IS_data<-data_noNA %>% filter(INPUT==IS_Cmp) 
  # Parsing column names and converting to numeric vector of calibration amounts
  cal_amounts<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), end=-3) %>% gsub("Cal","",.) %>% gsub("-",".",.) %>% unlist() %>% as.numeric()
  reps_list<-stringr::str_sub(native_data %>% select(starts_with('Cal')) %>% colnames(), start=-1) %>% as.numeric()
  # RFs for different types of models
  if(ratio){
    response_ratios<-(native_data %>% select(starts_with('Cal')))/(IS_data %>% select(starts_with('Cal'))) 
    df<-data.frame(native=rep(native_Cmp,length(cal_amounts)),IS=rep(IS_Cmp,length(cal_amounts)),replicate=reps_list,Exp.Amt=cal_amounts,Response.Ratio=as.numeric(response_ratios))
    df<-df[complete.cases(df),]
    df$RRF<-df$Response.Ratio/df$Exp.Amt
    return(df)
  }
  else{
    df<-data.frame(native=rep(native_Cmp,length(cal_amounts)),IS=rep(IS_Cmp,length(cal_amounts)),replicate=reps_list,Exp.Amt=cal_amounts,Area=native_data %>% select(starts_with('Cal')) %>% unlist() %>% as.numeric())
    df<-df[complete.cases(df),]
    df$RF<-df$Area/df$Exp.Amt
    return(df)
  }
}

### FUNCTIONS FOR BOOTSTRAPPING ### 
## BOOTSTRAPPING (FULL DATA OR LEAVE-ONE-OUT CROSS-VALIDATION (LOOCV))

# Function to repeat n=runs number of times to get many 2.5th, 50th, and 97.5th percentile estimates
# @param data_used: The data containing the full set of chemical RFs 
# @param training_data: The training set (the chemical RFs with the leave-one-out RFs excluded)
# @param resample_size: Set to the size of each bootstrap resample, otherwise the function defaults to resampling to the number of unique native compounds in the original full dataset
# @perc_out: If TRUE, return only the 2.5th, 50th, and 97.5th percentile values instead of the RFs sampled and chemicals sampled
perc_fun_new<-function(data_used,training_data,col_type="RF",resample_size=NA,perc_out=FALSE){
  # Hierarchical bootstrapping 
  n_chems<-length(unique(training_data$native))
  # Randomly sample one chemical (n*m chemicals)
  if(!is.na(resample_size)){
    chems_sampled<-sample(n_chems,size=resample_size,replace=T)
  }
  else{
    chems_sampled<-sample(n_chems,size=length(unique(data_used$native)),replace=T)
  }
  # if col_type="RRF", use RRF instead
  if(col_type=="RRF"){
    RFs_sampled<-sapply(chems_sampled,FUN=function(x) sample(training_data$RRF[training_data$native==unique(training_data$native)[x]],size=1))
  }
  else{
    # Randomly sample a RF value for each chemical
    RFs_sampled<-sapply(chems_sampled,FUN=function(x) sample(training_data$RF[training_data$native==unique(training_data$native)[x]],size=1))
  }
  if(perc_out){
    # Take the 2.5th and 97.5th percentile RF values
    perc_2.5th<-quantile(RFs_sampled,0.025)
    perc_50th<-quantile(RFs_sampled,0.50)
    perc_97.5th<-quantile(RFs_sampled,0.975)
    # return output data frame                                      
    return(data.frame(percentile_2.5th=perc_2.5th[[1]],percentile_50th=perc_50th[[1]],percentile_97.5th=perc_97.5th[[1]]))
  }
  else{
    return(data.frame(rf_sampled=RFs_sampled,chem_sampled=unique(training_data$native)[chems_sampled]))
  }
}

# Function which does leave-one-out cross-validation with hierarchical bootstrapping
# @param data_in: Dataframe containing columns for native compounds and RF values. Output from get_RF()
# @param col_type: Default is "RF". If col_type=="RRF", the function uses the column 'RRF' instead of 'RF'
# @param index: The index of the chemical serving as the leave-one-out chemical
# @param loocv: If TRUE, index is used to select a chemical from the chemicals column to serve as the leave-one-out training data chemical.
# @param runs: The number of runs for the bootstrap
# @param med_out: If TRUE, only output the median values of 2.5th and 97.5th percentiles instead of the entire dataframe
boot_fun_new<-function(data_in,col_type="RF",index=1,loocv=TRUE,runs,med_out=TRUE,resample_size=NA){
  # prepare the data
  cmps<-unique(data_in$native)
  # store the training data in a list of lists 
  training_df<-vector(mode="list",length=length(cmps))
  if(loocv){
    #test<-data_in[data_in$native==cmps[index],]
    training<-data_in[data_in$native!=cmps[index],]
  }
  else{
    training<-data_in
  }
  #pre-allocate a dataframe
  boot_percs<-data.frame(percentile_2.5th=double(runs),percentile_50th=double(runs),percentile_97.5th=double(runs))
  # Don't need columns storing all the sampled RFs and chemicals if only outputting the median percentile values
  if(!med_out){
    boot_percs$rf_sampled=vector("list",runs)
    boot_percs$chem_sampled=vector("list",runs)
  }
  
  # do the bootstrap with n=runs number of repeats
  for(i in 1:runs){
    temp<-perc_fun_new(data_in,training,col_type,resample_size)
    boot_percs$percentile_2.5th[i]<-quantile(temp$rf_sampled,0.025)[1]
    boot_percs$percentile_50th[i]<-quantile(temp$rf_sampled,0.50)[1]
    boot_percs$percentile_97.5th[i]<-quantile(temp$rf_sampled,0.975)[1]
    if(!med_out){
      boot_percs$rf_sampled[[i]]<-temp$rf_sampled
      boot_percs$chem_sampled[[i]]<-temp$chem_sampled
    }
  }
  if(med_out){
    return(data.frame(med_2.5th_percentile=median(boot_percs$percentile_2.5th),med_50th_percentile=median(boot_percs$percentile_50th),med_97.5th_percentile=median(boot_percs$percentile_97.5th)))
  }
  else{
    return(boot_percs)
  }
}

# Function which does bootstrap on the entire dataset with hierarchical bootstrapping
# @param data_in: Dataframe containing columns for native compounds and RF values. Output from get_RF()
# @param col_type: Default is "RF". If col_type=="RRF", the function uses the column 'RRF' instead of 'RF'
# @param index: The index of the chemical serving as the leave-one-out chemical
# @param runs: The number of runs for the bootstrap
# @param med_out: If TRUE, only output the median values of 2.5th and 97.5th percentiles instead of the entire dataframe
# @param list_cols: If TRUE, add to the output dataframe a list-type column with all the prediction errors for a given leave-one-out chemical

full_boot<-function(data_in,col_type="RF",index=1,runs,med_out=TRUE,list_cols=FALSE,resample_size=NA){
  # pre-allocate dataframe 
  full_boot_out<-data.frame(min_2.5th_percentile=double(1),med_2.5th_percentile=double(1),max_2.5th_percentile=double(1),min_50th_percentile=double(1),med_50th_percentile=double(1),max_50th_percentile=double(1),min_97.5th_percentile=double(1),med_97.5th_percentile=double(1),max_97.5th_percentile=double(1))
  boot_out<-boot_fun_new(data_in,col_type,index=1,loocv=FALSE,runs,med_out,resample_size)
  # if only medians were output
  if(med_out){
    full_boot_out$med_2.5th_percentile<-boot_out$med_2.5th_percentile[1]
    full_boot_out$med_50th_percentile<-boot_out$med_50th_percentile[1]
    full_boot_out$med_97.5th_percentile<-boot_out$med_97.5th_percentile[1]
  }
  # if full boot_percs dataframe was output
  else{
    full_boot_out$min_2.5th_percentile<-min(boot_out$percentile_2.5th)
    full_boot_out$med_2.5th_percentile<-median(boot_out$percentile_2.5th)
    full_boot_out$max_2.5th_percentile<-max(boot_out$percentile_2.5th)
    full_boot_out$min_50th_percentile<-min(boot_out$percentile_50th)
    full_boot_out$med_50th_percentile<-median(boot_out$percentile_50th)
    full_boot_out$max_50th_percentile<-max(boot_out$percentile_50th)
    full_boot_out$min_97.5th_percentile<-min(boot_out$percentile_97.5th)
    full_boot_out$med_97.5th_percentile<-median(boot_out$percentile_97.5th)
    full_boot_out$max_97.5th_percentile<-max(boot_out$percentile_97.5th)
  }
  return(full_boot_out)
}

# Function which does leave-one-out cross-validation with hierarchical bootstrapping
# @param data_in: Dataframe containing columns for native compounds and RF values. Output from get_RF()
# @param col_type: Default is "RF". If col_type=="RRF", the function uses the column 'RRF' instead of 'RF'
# @param index: The index of the chemical serving as the leave-one-out chemical
# @param runs: The number of runs for the bootstrap
# @param med_out: If TRUE, only output the median values of 2.5th and 97.5th percentiles instead of the entire dataframe
# @param list_cols: If TRUE, add to the output dataframe a list-type column with all the prediction errors for a given leave-one-out chemical

loocv_new<-function(data_in,col_type="RF",index=1,runs,med_out=TRUE,list_cols=FALSE){
  cmps<-unique(data_in$native)
  n<-length(cmps)
  # pre-allocate dataframe 
  loocv_out<-data.frame(min_2.5th_percentile=double(n),med_2.5th_percentile=double(n),max_2.5th_percentile=double(n),min_50th_percentile=double(n),med_50th_percentile=double(n),max_50th_percentile=double(n),min_97.5th_percentile=double(n),med_97.5th_percentile=double(n),max_97.5th_percentile=double(n),perc_out_lower=double(n),perc_in=double(n),perc_out_upper=double(n))
  for(i in 1:n){
    boot_out<-boot_fun_new(data_in,col_type,index=i,loocv=TRUE,runs,med_out)
    # if only medians were output
    if(med_out){
      loocv_out$med_2.5th_percentile[i]<-boot_out$med_2.5th_percentile[1]
      loocv_out$med_50th_percentile[i]<-boot_out$med_50th_percentile[1]
      loocv_out$med_97.5th_percentile[i]<-boot_out$med_97.5th_percentile[1]
    }
    # if full boot_percs dataframe was output
    else{
      loocv_out$min_2.5th_percentile[i]<-min(boot_out$percentile_2.5th)
      loocv_out$med_2.5th_percentile[i]<-median(boot_out$percentile_2.5th)
      loocv_out$max_2.5th_percentile[i]<-max(boot_out$percentile_2.5th)
      loocv_out$min_50th_percentile[i]<-min(boot_out$percentile_50th)
      loocv_out$med_50th_percentile[i]<-median(boot_out$percentile_50th)
      loocv_out$max_50th_percentile[i]<-max(boot_out$percentile_50th)
      loocv_out$min_97.5th_percentile[i]<-min(boot_out$percentile_97.5th)
      loocv_out$med_97.5th_percentile[i]<-median(boot_out$percentile_97.5th)
      loocv_out$max_97.5th_percentile[i]<-max(boot_out$percentile_97.5th)
    }
    test_rfs<-data_in[data_in$native==cmps[i],c(col_type)]
    loocv_out$perc_out_lower[i]<-(length(which(test_rfs<loocv_out$med_2.5th_percentile[i])))/length(test_rfs)
    loocv_out$perc_in[i]<-(length(which(test_rfs>loocv_out$med_2.5th_percentile[i] & test_rfs<loocv_out$med_97.5th_percentile[i])))/length(test_rfs)
    loocv_out$perc_out_upper[i]<-(length(which(test_rfs>loocv_out$med_97.5th_percentile[i])))/length(test_rfs)
  }
  return(loocv_out)
}
